@@ -13,7 +13,7 @@ use Server::Starter ();
 use Try::Tiny;
 use Guard;
 
-our $VERSION = "0.23";
+our $VERSION = "0.25";
 
 use XSLoader;
 XSLoader::load(__PACKAGE__, $VERSION);
@@ -123,6 +123,7 @@ sub run {
 
             my $proc_req_count = 0;
             $self->{can_exit} = 1;
+            $self->{term_received} = 0;
             local $SIG{TERM} = sub {
                 exit 0 if $self->{can_exit};
                 $self->{term_received}++;
@@ -206,10 +207,11 @@ sub run {
                     } else {
                         die "Bad response $res";
                     }
-                    if ($self->{term_received} || $env->{'psgix.harakiri.commit'}) {
+                    if ($env->{'psgix.harakiri.commit'}) {
                         exit 0;
                     }
                 }
+                exit 0 if $self->{term_received};
             }
         });
     }
